#!/usr/bin/env python
import os
import subprocess
import sys
import tarfile
import shlex
import shutil
import inspect

from charmhelpers.lib.utils import config_get
from shutil import rmtree, copyfile
from socket import gethostname
from charmhelpers.core.hookenv import log, Hooks, relation_get, relation_set, unit_get, open_port
from charmhelpers.core.host import service_start, service_stop
from time import sleep
# Required for unit tests... :(
try:
    from charmhelpers.fetch import apt_install, apt_update,  apt_purge
except:
    pass

def append_bashrc(line):
    log("==> {}".format(inspect.stack()[0][3]),"INFO")
    with open(os.path.join(os.path.sep, home, '.bashrc'),'a') as bf:
        bf.writelines(line)
        
def setHadoopEnvVar(scriptName):
    log("==> Enter {}".format(inspect.stack()[0][3]),"INFO")
    with open(scriptName) as f:
        lines = f.readlines()
        for l in lines:
            if l.startswith("#") or l == '\n':
                continue
            else:
                ll = l.split("=")
                m = ll[0]+" = "+ll[1].strip().strip(';').strip("\"")
                log("==> setHadoopEnvVar : {}".format(m))
                os.environ[ll[0]] = ll[1].strip().strip(';').strip("\"")
                
                                
def setHadoopConfigXML (xmlfileName, name, value):
    log("==> setHadoopConfigXML ","INFO")
    import xml.etree.ElementTree as ET
    found = False
    xmlfileNamePath = os.path.join(os.path.sep, os.environ["HADOOP_CONF_DIR"], xmlfileName)
    with open(xmlfileNamePath,'rb+') as f:    
        root = ET.parse(f).getroot()
        proList = root.findall("property")
        for p in proList:
            cList = p.getchildren()
            for c in cList:
                if c.text == name:
                    p.find("value").text = value
                    found = True
        if found:
            f.seek(0)
            f.write(ET.tostring(root, encoding='UTF-8'))
            f.truncate()
                
    return found  
        
# Parses /proc/meminfo for total memory in GB
def get_system_total_memory():
    log("==> {}".format(inspect.stack()[0][3]),"INFO")
    with open("/proc/meminfo") as f:
        lines = f.readlines()
        for i in range(0, len(lines)):
            if lines[i].startswith("MemTotal"):
                n = int (lines[i].split()[1])
                return str(n/1000000)      
   
# Parses /proc/cpuinfo for number of physical cores          
def get_system_cores():
    log("==> {}".format(inspect.stack()[0][3]),"INFO")
    with open("/proc/cpuinfo") as f:
        lines = f.readlines()
        for i in range(0, len(lines)):
            if lines[i].startswith("cpu cores"):
                return str(lines[i].split()[3])
def get_system_disk():
    log("==> {}".format(inspect.stack()[0][3]),"INFO")
    #TODO must get from config.yaml
    return str(1)

def is_hbase_installed():
    log("==> {}".format(inspect.stack()[0][3]),"INFO")
    #TODO must get from config.yaml
    return "False"
         
def fileSetKV(filePath, key, value):
    log ("===> fileSetKV ({}, {}, {})".format(filePath, key, value))
    with open(filePath) as f:
        contents = f.readlines()
        for l in range(0, len(contents)):
            if contents[l].startswith(key):
                contents[l] = key+value
             
    with open(filePath, 'w') as f:
        f.writelines(contents)
                
def wgetPkg(pkgName, crypType):
    log("==> {}".format(inspect.stack()[0][3]),"INFO")
    crypFileName= pkgName+'.'+crypType    
    cmd = 'wget '+pkgName
    subprocess.call(cmd.split())
    if crypType:
        cmd = ['wget', crypFileName]
        subprocess.call(cmd)
    # TODO - must add incryption  ver
    
def calculateHDPOptMemConfSetting():
    log("==> {}".format(inspect.stack()[0][3]),"INFO")
    os.chdir(hdpScriptPath)
    cmd = "python hdp-configuration-utils.py -c "+get_system_cores()+" -m "+\
           get_system_total_memory()+" -d "+get_system_disk()+" -k "+is_hbase_installed()+\
           " >"+hadoopMemoryOptimizationData
    log (" cmd ="+cmd, "info")
    subprocess.check_output(cmd.split())
            
# Update HDP directories.sh file with user defined configuration defined
# in config.yaml
def updateDirectoriesScript():
    log("==> {}".format(inspect.stack()[0][3]),"INFO")
    fileSetKV(directoriesScript, "DFS_NAME_DIR=", "\""+config_get('dfs_name_dir')+"\";")
    fileSetKV(directoriesScript, "DFS_DATA_DIR=","\""+ config_get('dfs_data_dir')+"\";")
    fileSetKV(directoriesScript, "FS_CHECKPOINT_DIR=", "\""+config_get('fs_checkpoint_dir')+"\";")
    fileSetKV(directoriesScript, "YARN_LOCAL_DIR=", "\""+config_get('yarn_local_dir')+"\";")
    fileSetKV(directoriesScript, "YARN_LOCAL_LOG_DIR=", "\""+config_get('yarn_local_log_dir')+"\";")
    fileSetKV(directoriesScript, "ZOOKEEPER_DATA_DIR=", "\""+config_get('zookeeper_data_dir')+"\";") 
    fileSetKV(directoriesScript, "HADOOP_CONF_DIR=", "\""+"/etc/hadoop/conf.juju"+"\";")
#uninstall installed packages
def createHDPHadoopConf():
    log("==> {}".format(inspect.stack()[0][3]),"INFO")
   
    HADOOP_CONF_DIR = os.environ["HADOOP_CONF_DIR"]
    HDPConfPath = os.path.join(os.path.sep,home, hdpScript, "configuration_files", "core_hadoop")
    source = os.listdir(HDPConfPath)
    os.mkdir(HADOOP_CONF_DIR)
    for files in source:
        srcFile = os.path.join(os.path.sep, HDPConfPath, files)
        desFile = os.path.join(os.path.sep, HADOOP_CONF_DIR, files)
        shutil.copyfile(srcFile, desFile)
    subprocess.call(createDadoopConfDir)
     
def install_base_pkg():
    log("==> {}".format(inspect.stack()[0][3]),"INFO")
    log("==> install_base_pkg", "INFO")
    wgetPkg("http://public-repo-1.hortonworks.com/HDP/ubuntu12/2.1.3.0/hdp.list -O /etc/apt/sources.list.d/hdp.list","")
    
    cmd =gpg_script 
    subprocess.call(cmd)   
    packages = ['ntp',
                'openjdk-7-jdk',
                'hadoop',
                'hadoop-hdfs',  
                'hadoop-yarn',
                'hadoop-mapreduce',
                'hadoop-client',
                'openssl',
                'libsnappy1',
                'libsnappy-dev',
                'liblzo2-2',
                'liblzo2-dev',
                'libhdfs0',
                'libhdfs0-dev',
                'hadoop-lzo']
    apt_update()
    apt_install(packages)
    os.chdir(home);
    wgetPkg("http://public-repo-1.hortonworks.com/HDP/tools/2.1.1.0/hdp_manual_install_rpm_helper_files-2.1.1.385.tar.gz","")
    if tarfile.is_tarfile(tarfilename):
        tball = tarfile.open(tarfilename)
        tball.extractall(home)
    else:
        log ("Unable to extract Hadoop Tarball ", "ERROR")
    os.rename(tarfilenamePre, hdpScript)
    log("<== install_base_pkg", "INFO")
  
def configureJAVA(installedJavaPath):
    log("==> {}".format(inspect.stack()[0][3]),"INFO")
    javaPath = os.path.join(os.path.sep, "usr", "java")
    javaDefault = os.path.join(os.path.sep, javaPath, "default")
    javaDefaultBinPath = os.path.join(os.path.sep, javaDefault, "bin", "java")
    javaBinPath = os.path.join(os.path.sep, "usr", "bin", "java")
    os.mkdir(javaPath)
    os.symlink(installedJavaPath, javaDefault)
   # TBD os.symlink(javaDefaultBinPath, javaBinPath)
    append_bashrc("export JAVA_HOME={}\n".format(javaDefault))
    append_bashrc("export PATH=$JAVA_HOME/bin:$PATH\n")
        
    
def uninstall_base_pkg():
    log("==> {}".format(inspect.stack()[0][3]),"INFO")
    packages = ['ntp',
                'openjdk-7-jdk',
                'hadoop',
                'hadoop-hdfs', 
                'hadoop-yarn',
                'hadoop-mapreduce',
                'hadoop-client',
                'openssl',
                'libsnappy1',
                'libsnappy-dev',
                'liblzo2-2',
                'liblzo2-dev',
                'libhdfs0',
                'libhdfs0-dev',
                'hadoop-lzo']
    apt_purge(packages)
    shutil.rmtree(hdpScriptPath)
    shutil.copyfile(bashrc+".org", bashrc)
    
def config_all_nodes():
    log("==> {}".format(inspect.stack()[0][3]),"INFO")
    #TODO All hosts in your system must be configured for DNS and Reverse DNS.
    service_start("ntp")
    service_stop( "ufw")
    #backyp bashrc file
    if not os.path.isfile(bashrc+".org"):
        shutil.copyfile(bashrc, bashrc+".org")
    append_bashrc("umask 022\n")
    #sourcing hadoop user ids and groups
    append_bashrc("source "+usersAndGroupsScript+'\n')
    updateDirectoriesScript()
    #sourcing hadoop directory structure 
    append_bashrc("source "+directoriesScript+'\n')
    setHadoopEnvVar(usersAndGroupsScript)
    setHadoopEnvVar(directoriesScript)
    log("==> {}".format(inspect.stack()[0][3])+" calling subprocess.call(hdp_hellper_script)","INFO")
    subprocess.call(hdp_hellper_script)
    calculateHDPOptMemConfSetting()
    configureJAVA(javaPath)
    createHDPHadoopConf()
    subprocess.call(createLogPIDallNodes)
    subprocess.call(createResourceManDataNodeDir)
    
    
    
    
    
home  = os.path.join(os.path.sep, "home", "ubuntu") 
javaPath = "/usr/lib/jvm/java-1.7.0-openjdk-amd64"
# Below is an Ubuntu specific gpg script file
# must change for other platform 
HDP_PGP_SCRIPT = 'gpg_ubuntu.sh'
HADOOP_CONF_DIR = os.path.join (os.path.sep, )
gpg_script = os.path.join(os.path.sep, os.environ['CHARM_DIR'], os.path.sep, os.environ['CHARM_DIR'], 'files', 'scripts',HDP_PGP_SCRIPT)
hdp_hellper_script = os.path.join(os.path.sep, os.environ['CHARM_DIR'],'files', 'scripts', "HDPHelperScripts.sh")
if not os.path.isfile(hdp_hellper_script):
    log ("Erro ==> {} not found".format(hdp_hellper_script), "ERROR")
createLogPIDallNodes = os.path.join(os.path.sep, os.environ['CHARM_DIR'],'files', 'scripts', "createLogPIDallNodes.sh")
createDadoopConfDir = os.path.join(os.path.sep, os.environ['CHARM_DIR'],'files', 'scripts', "createHadoopConfDir.sh")
createResourceManDataNodeDir = os.path.join(os.path.sep, os.environ['CHARM_DIR'],'files', 'scripts', "createResourceManDataNodeDir.sh")
hdpScript = "hdp_scripts"
hdpScriptPath = os.path.join(os.path.sep,home, hdpScript,'scripts')
usersAndGroupsScript = os.path.join(os.path.sep, hdpScriptPath, "usersAndGroups.sh")
directoriesScript =  os.path.join(os.path.sep, hdpScriptPath, "directories.sh")
tarfilename="hdp_manual_install_rpm_helper_files-2.1.1.385.tar.gz"
tarfilenamePre="hdp_manual_install_rpm_helper_files-2.1.1.385"
bashrc = os.path.join(os.path.sep, home, '.bashrc')
hadoopMemoryOptimizationData = os.path.join(os.path.sep, hdpScriptPath, "hdpMemOpt.txt");
nodeType="unknown"
nameNodeReady = False
ResourceManagerReady = False


# Start the Hook Logic Block
hooks = Hooks()


@hooks.hook('install')
def install():
    install_base_pkg()
    config_all_nodes()
        
    
@hooks.hook('resourcemanager-relation-joined')
def resourcemanager_relation_joined():
    log ("resourcemanager-relation-joined","INFO")
    nodeType="resourcemanager"
    

@hooks.hook('nodemanager-relation-changed')
def nodemanager_relation_changed():
    log ("nodemanager_relation_changed","INFO")
          
@hooks.hook('resourcemanager-relation-broken')
def resourcemanager_relation_broken():
    log ("Configuring resourcemanager - broken phase","INFO")
      

@hooks.hook('namenode-relation-joined')
def namenode_relation_joined():
    log("Configuring namenode - joined phase", "INFO")
    nodeType="namenode"
    createNameNodeDirs = os.path.join(os.path.sep, os.environ['CHARM_DIR'],'files',\
                                       'scripts', "createNameNodeDirs.sh")
    subprocess.call(createNameNodeDirs)
    relation_set(namenode_hostname=gethostname())
    setHadoopConfigXML("core-site.xml", "fs.defaultFS", gethostname())
    
    
    sleep(5)
    nameNodeReady = True
    
    

@hooks.hook('datanode-relation-changed')
def datanode_relation_changed():
    log("Configuring datanode - Changed phase","INFO")
    if not nameNodeReady:
        sys.exit(0)
    nodeType="namenode"
    namenode_hostname = relation_get("namenode_hostname")
    setHadoopConfigXML("core-site.xml", "fs.defaultFS", namenode_hostname)
    createDataNodeDirs = os.path.join(os.path.sep, os.environ['CHARM_DIR'],'files',\
                                       'scripts', "createDataNodeDirs.sh")
    subprocess.call(createDataNodeDirs)
    
        
@hooks.hook('config-changed')
def config_changed():     
    log( "config-changed called", "INFO")  

@hooks.hook('namenode-relation-changed')

def namenode_relation_changed():
    log("Configuring namenode - changed phase", "INFO")     

if __name__ == "__main__":
    hooks.execute(sys.argv)